#include "PreferenceWriter.h"
#include "FlashWriter.h"
#include "user_config.h"
#include "mbed.h"

PreferenceWriter::PreferenceWriter(uint32_t sector) {
    writer = new FlashWriter(sector);
    __sector = sector;
    __ready = false;
}

void PreferenceWriter::open() {
    writer->open();
    __ready = true;
}

bool  PreferenceWriter::ready() {
    return __ready;
}

void PreferenceWriter::write(int x, int index) {
    __int_reg[index] = x;
}

void PreferenceWriter::write(float x, int index) {
    __float_reg[index] = x;
}

void PreferenceWriter::flush() {
    int offs;

    /* 
    //----- origin -----
    for (offs = 0; offs < 256; offs++) {
        writer->write(offs, __int_reg[offs]);
    }
    
    for (; offs < 320; offs++) {
        writer->write(offs, __float_reg[offs - 256]);
    }
    //---------------------
    */
    
    /*===== add calibration direction to flash memory =====*/
    for (offs = 0; offs < 257; offs++) {
        writer->write(offs, __int_reg[offs]);
    }
    
    /*===== add kp, ki, kd to flash memory =====*/
    for (; offs < 326; offs++) {
        writer->write(offs, __float_reg[offs - 257]);
    }
    __ready = false;
}

void PreferenceWriter::load() {
    int offs;
    /*
    // ----- origin -----
    for (offs = 0; offs < 256; offs++) {
        __int_reg[offs] = flashReadInt(__sector, offs);
    }
    
    for(; offs < 320; offs++) {
        __float_reg[offs - 256] = flashReadFloat(__sector, offs);
    }
    // ------------------
    */
    
    /*===== add calibration direction to flash memory =====*/
    for (offs = 0; offs < 257; offs++) {
        __int_reg[offs] = flashReadInt(__sector, offs);
    }
    
    /*===== add kp, ki, kd to flash memory =====*/
    for(; offs < 326; offs++) {
        __float_reg[offs - 257] = flashReadFloat(__sector, offs);
    }   
}

void PreferenceWriter::close() {
    __ready = false;
    writer->close();
}